﻿# Commands and Events Axon Example

The business requires a small program that keeps track of tasks.  Users can add notes to the tasks as long as these are still open.  No more notes can be attached to the task once the task is complete.  Duplicate notes should not be allowed either.

## Technologies Used

This example uses the following AXON technologies

1. Aggregates
2. Commands (and Command Handlers)
3. Events (and Event Handlers)
4. Command Gateway

The project is built using Maven and requires Java 8 or newer.  The events are persisted within a MySQL database as documented next.

### Database

The demo persists data, such as events, within an RDBMS database.  There are other options, such as MongoDB, but RDBMS is the most common and available option.  The demo requires an empty MySQL database named `test_axon` to be available and a username `test` with password `test` as defined in the `application.yml` file.  Modify these properties as needed.

## Implementation

The business requires a small program that manages tasks and their notes.  The task business object will be represented by an aggregate, named `Task`.  The aggregate is responsible from managing the state of each task and will host the command handlers (methods that will process the commands) that will modify the aggregate's state.  A set of commands will be used to create the task (such as `CreateTaskCommand`), complete it and add notes to it.  Each command will publish one or more events which events are then used to update the same aggregate state.


For example, the command `CreateTaskCommand` is used to create a new task aggregate.  This command is handled by the `Task` aggregate constructor as shown in the following fragment.

```
@Aggregate
public class Task {

  @AggregateIdentifier
  private TaskId id;
  private TaskStage stage;
  private Set<TaskNote> notes;

  @SuppressWarnings("unused")
  private Task() {}

  @CommandHandler
  public Task(final CreateTaskCommand command) {
    AggregateLifecycle.apply(TaskCreatedEvent.of(command));
  }

  /* Other methods removed for brevity */
}
```

Please note that parts of the class are shown here for brevity.  The state of an aggregate is not modified by the command handler, but by the events handlers that process the events produced by the same command as shown in the following method from the same `Task` aggregate class.

```
@EventSourcingHandler
public void apply(final TaskCreatedEvent event) {
  id = event.getId();
  stage = TaskStage.OPEN;
  notes = new LinkedHashSet<>();
}
```

All aggregate state is updated and initialised as needed at the event handler.  This is imperative as next time this particular task is needed, AXON will load all of its events and will replay them one after the other, invoking one event handler after the other passing the same events to these methods.

An aggregate can have more than one command handler method and/or event handler method.  A task can be completed using the `CompleteTaskCommand` command, which handler is shown next.


```
@CommandHandler
public void handle(final CompleteTaskCommand command) {
  if (stage == TaskStage.OPEN) {
    AggregateLifecycle.apply(TaskCompletedEvent.of(command));
  }
}
```

The above method is too found within the `Task` aggregate.  Note how this command handler first checks the task stage and only publish the `TaskCompletedEvent` event if the task is still open.  The command handler does nothing in the event the task was already completed.  Invoking the `CompleteTaskCommand` will have no effect once the task is marked complete.

```
@EventSourcingHandler
public void apply(final TaskCompletedEvent event) {
  stage = TaskStage.COMPLETE;
}
```

The event handler for the event `TaskCompletedEvent` changes the task's stage state to complete unconditionally.  This is another important differentiation between commands and events.  Command handlers are responsible from the business logic, while events simply follow orders.

Events represent the past, something that no one can change.  What if we do a mistake, some may ask.  In such event we publish corrective events and not try to modify the past.  Say, for example, you withdraw €100 instead of €10 by mistake from your account.  You simple take a corrective action and deposit €90 back into your account.  If you look at your transactions at the end of the month you will see that you withdrew €100 and then deposit back €90, leaving you with €10 in your pocket as originally planned.  The bank will not merge the first withdrawal and with the second deposit into one event, as that's wrong.  Same thing happened with the events.  In the event of a mistake we take a corrective action and publish new events that correct the first mistake.

Back to our problem now.  We have a final command to implement.  We need to be able to add notes to the `Task` aggregate but only if the task is still open and only if this note does not exists.  The command handler needs to check for these conditions before publishing the proper event.

```
@CommandHandler
public AddNoteResult handle(final AddNoteToTaskCommand command) {
  if (stage == TaskStage.COMPLETE) {
    AggregateLifecycle.apply(TaskAlreadyCompletedEvent.of(command));
    return AddNoteResult.ALREADY_COMPLETE;
  }

  if (notes.contains(command.getNote())) {
    AggregateLifecycle.apply(NoteAlreadyExistsInTaskEvent.of(command));
    return AddNoteResult.ALREADY_EXISTS;
  }

  AggregateLifecycle.apply(NoteAddedToTaskEvent.of(command));
  return AddNoteResult.ADDED;
}
```

Note that the above command handler returns a value.  Our program will be connected to a user interface through which the users will interact and manage tasks.  In such case we need to show the user what happened and provide the proper feedback, such as 'note added' or 'note was not added as the task is closed' or take a different action.  The command handlers can tell us what happened by returning values.  In other words, together with publishing events, the command handlers can also return values which the caller can use.

How can we use the values returned by a command handler?  The `CommandGateway`, another component of Axon, allows us to provide a `CommandCallback` through which we can retrieve the command handler's result.  In case of `void`, the `CommandCallback` will return `null` as the result.

```
/* Used to simply document the outcome of the add note command */
final CommandCallback<AddNoteToTaskCommand, AddNoteResult> callback = new CommandCallback<AddNoteToTaskCommand, AddNoteResult>() {
  @Override
  public void onFailure(final CommandMessage<? extends AddNoteToTaskCommand> commandMessage, final Throwable cause) {}

  @Override
  public void onSuccess(final CommandMessage<? extends AddNoteToTaskCommand> commandMessage, final AddNoteResult result) {
    Demo.LOGGER.debug("Result: {}", result);
  }
};

/* Add a note */
gateway.send(AddNoteToTaskCommand.of(id, TaskNote.of("Note 1")), callback);
```

This concludes the simple demo demonstrating the use of aggregates, command, events and command gateway.  The business (or domain) objects are represented by aggregates.  The command handlers are responsible form the business logic and will publish events to change the aggregate's state.  The command handlers may or may not return values to interact with the caller.  The aggregate's state is only modified by the event handlers processing the events handlers and never by the command handlers.  The event handlers should simply do what they are told to do.
